//
//  line.js
//  Library
//
//  Created by Jonathon Mah on 2013-03-19.
//  Copyright (c) 2013 Delicious Monster Software. All rights reserved.
//

var Line_unique = 1;


function Line(chartDiv, svg, width, height) {
    this.chartDiv = chartDiv;
    chartDiv.classed("line-chart", true);
    this.svg = svg;
    this.width = width;
    this.height = height;

    this.axisSize = [48, 18]; // x,y space to show axis and labels
    this.axisPadding = [7, 5]; // extra space to avoid cutting off labels

    this.xScale = d3.scale.linear();
    this.yScale = d3.scale.linear();

    this.xAxis = d3.svg.axis()
        .scale(this.xScale)
        .ticks(6)
        .orient("bottom");

    this.yAxis = d3.svg.axis()
        .scale(this.yScale)
        .orient("left")
        .ticks(5);

    var self = this;
    this.line = d3.svg.line()
        .interpolate("basis")
        .x(function(d) { return self.xScale(d[0]); })
        .y(function(d) { return self.yScale(d[1]); });

    var clippingID = "line-clip-" + Line_unique++;
    this.clip =
        this.svg
        .append("clipPath")
        .attr("id", clippingID)
        .append("rect");

    this.svg
        .append("path")
        .classed("line", true)
        .attr("clip-path", "url(#" + clippingID + ")");
}


Line.prototype.update = function() {
    var self = this;

    var values = this.dataFunction();

    var xRange = [this.axisSize[0], this.width - this.axisPadding[0]];
    var yRange = [this.height - this.axisSize[1], this.axisPadding[1]];
    this.xScale.range(xRange);
    this.yScale.range(yRange);

    // OPT: max is accumulator, if .cumulative
    this.yScale
        .domain([0, d3.max(values, function(d) { return d[1]; })])
        .nice();

    this.clip
        .attr("x", xRange[0])
        .attr("width", 0)
        .attr("y", yRange[1])
        .attr("height", yRange[0] - yRange[1]);

    this.svg
        .select("path.line")
        .datum(values)
        .attr("d", this.line);


    var animateFromScratch = true;
    var newYScaleDomain = this.yScale.domain();
    if (this.lastYScaleDomain)
        // only animate line drawing if Y axis changes
        animateFromScratch = (this.lastYScaleDomain[0] != newYScaleDomain[0] ||
                              this.lastYScaleDomain[1] != newYScaleDomain[1]);
    this.lastYScaleDomain = newYScaleDomain;

    if (animateFromScratch) {
        this.clip
            .transition()
            .delay(250)
            .duration(750)
            .ease(d3.ease("linear"))
            .attr("width", xRange[1] - xRange[0]);
    } else {
        this.clip
            .attr("width", xRange[1] - xRange[0]);
    }


    // Axes
    var xAxisElem = this.svg.select("g.x.axis");
    if (xAxisElem.empty()) {
        xAxisElem =
            this.svg
            .append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(0," + (self.height - self.axisSize[1]) + ")")
            .call(this.xAxis);
    } else {
        xAxisElem
            .transition()
            .duration(750)
            .call(this.xAxis);
    }

    var yAxisElem = this.svg.select("g.y.axis");
    if (yAxisElem.empty()) {
        yAxisElem =
            this.svg
            .append("g")
            .attr("class", "y axis")
            .attr("transform", "translate(" + self.axisSize[0] + ",0)")
            .call(this.yAxis);
    } else {
        yAxisElem
            .transition()
            .duration(750)
            .call(this.yAxis);
    }
};
